@@ -1,6 +1,6 @@
 ;; language/nx-mlang/spec.scm - NYACC extension for Octave
 
-;; Copyright (C) 2018 Matthew R. Wette
+;; Copyright (C) 2018 Matthew Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -40,7 +40,6 @@
                 (lambda ()
                   (let ((env (make-fresh-user-module)))
                     (module-define! env 'current-reader (make-fluid))
-                    env))
-  )
+                    env)))
 
 ;; --- last line ---
