@@ -1,6 +1,6 @@
 ;;; nyacc/lang/c99/ffi-help.scm
 
-;; Copyright (C) 2016-2026 Matthew Wette
+;; Copyright (C) 2016-2025 Matthew Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -64,7 +64,7 @@
   #:use-module (nyacc foreign arch-info)
   #:use-module (nyacc lang c99 fh-utils)
   #:re-export (*nyacc-version*)
-  #:version (3 03 0))
+  #:version (3 02 0))
 
 (define default-renamer (lambda (name ctxt) name))
 
@@ -382,9 +382,7 @@
 
 (define (cdata-header)
   `(begin
-     ,(if (assq-ref (*options*) 'ext-cdata)
-         '(use-modules (foreign cdata))
-         '(use-modules (nyacc foreign cdata)))
+     (use-modules (nyacc foreign cdata))
      (define arg->number cdata-arg->number)
      (define arg->pointer cdata-arg->pointer)
      (define (extern-ref obj) (cdata-sel obj '*))
@@ -881,14 +879,19 @@
 
 ;; @deffn {Procedure} udecl->sexp udecl udict defined seed)
 ;; Given @var{udecl} produce scheme FFI wrappers for C types, C functions,
-;; and C variables. Return two values: a list of defined declarations, and
-;; a list of the converted declarations.
+;; and C variables. Return updated @var{defined}, a string based vhash of
+;; types defined. The list is used used in the conversion subroutines.
+;; Returns (values defined term), where defined is a list of defined
+;; types, and wrapped is the same, not used. FIXME
 ;; @end deffn
 (define (udecl->sexp udecl udict defined-lz seed)
+  ;; this used to be cnvt-udecl
   "- Procedure: udecl->sexp udecl udict defined seed)
      Given UDECL produce scheme FFI wrappers for C types, C functions,
-     and C variables.  Return two values: a list of defined
-     declarations, and a list of the converted declarations."
+     and C variables.  Return updated DEFINED, a string based vhash of
+     types defined.  The list is used used in the conversion
+     subroutines.  Returns (values defined term), where defined is a
+     list of defined types, and wrapped is the same, not used.  FIXME"
 
   (define defined (if (vhash? defined-lz) defined-lz
                       (alist->vhash (map (lambda (n) (cons n #t)) defined-lz))))
@@ -1321,8 +1324,8 @@
          (dirs (if pkg-config (append (pkg-config-dirs pkg-config) dirs) dirs)))
     (sfscm ";; generated with `guild compile-ffi ~A.ffi'\n"
            (m-path->f-path path))
-    (sfscm ";; using nyacc version ~a for ~a\n"
-           *nyacc-version* (arch-name (*arch*)))
+    (sfscm ";; using nyacc version ~a\n" *nyacc-version*)
+
     (nlscm)
     (sfscm "(define-module ~S\n" path)
     (for-each ;; ffi-modules
@@ -1388,7 +1391,7 @@
         (defs
           (fold
            (lambda (name seed)
-             (let* ((repl (macro-expand-text name ddict))
+             (let* ((repl (expand-cpp-name name ddict '()))
                     (val (and (string? repl)
                               (false-if-exception (eval-code-string repl)))))
                (if val
