@@ -114,6 +114,7 @@
      ("sizeof" unary-expression ($$ `(sizeof-expr ,$2)))
      ("sizeof" "(" type-name ")" ($$ `(sizeof-type ,$3)))
      ("_Alignof" "(" type-name ")" ($$ `(alignof-type ,$3)))
+     ;; S-R conflict ("__alignof__" "(" type-name ")" ($$ `(alignof-type ,$3)))
      ("__builtin_offsetof" "(" type-name "," constant-expression ")"
       ($$ `(offsetof-type ,$3 ,$5)))
      )
@@ -278,8 +279,8 @@
      ("signed" "short" ($prec 'imp) ($$ '(fixed-type "short")))
      ("signed" "short" "int" ($$ '(fixed-type "short")))
      ("int" ($$ '(fixed-type "int")))
-     ("signed" ($prec 'imp) ($$ '(fixed-type "signed")))
-     ("signed" "int" ($$ '(fixed-type "signed")))
+     ("signed" ($prec 'imp) ($$ '(fixed-type "int")))
+     ("signed" "int" ($$ '(fixed-type "int")))
      ("long" ($prec 'imp) ($$ '(fixed-type "long")))
      ("long" "int" ($$ '(fixed-type "long")))
      ("signed" "long" ($prec 'imp) ($$ '(fixed-type "long")))
@@ -303,6 +304,9 @@
      ("unsigned" "char" ($$ '(fixed-type "unsigned char")))
      ("__int128" ($$ '(fixed-type "__int128")))
      ("unsigned" "__int128" ($$ '(fixed-type "unsigned __int128")))
+     ;; gnu extensions
+     ("__int128_t" ($$ '(fixed-type "__int128")))
+     ("unsigned __int128_t" ($$ '(fixed-type "unsigned __int128")))
      ;; fixes 250627
      ("short" "signed" ($prec 'imp) ($$ '(fixed-type "short")))
      ("short" "signed" "int" ($$ '(fixed-type "short")))
@@ -353,15 +357,15 @@
     ;; This one modified: split out struct-or-union = "struct"|"union"
     (struct-or-union-specifier
      ("struct" opt-attr-specs ident-like "{" struct-declaration-list "}"
-      ($$ (sx-list 'struct-def $2 $3 (tl->list $5))))
+      ($$ (sx-list 'struct-def $2 $3 $5)))
      ("struct" opt-attr-specs "{" struct-declaration-list "}"
-      ($$ (sx-list 'struct-def $2 (tl->list $4))))
+      ($$ (sx-list 'struct-def $2 $4)))
      ("struct" opt-attr-specs ident-like
       ($$ (sx-list 'struct-ref $2 $3)))
      ("union" opt-attr-specs ident-like "{" struct-declaration-list "}"
-      ($$ (sx-list 'union-def $2 $3 (tl->list $5))))
+      ($$ (sx-list 'union-def $2 $3 $5)))
      ("union" opt-attr-specs "{" struct-declaration-list "}"
-      ($$ (sx-list 'union-def $2 (tl->list $4))))
+      ($$ (sx-list 'union-def $2 $4)))
      ("union" opt-attr-specs ident-like
       ($$ (sx-list 'union-ref $2 $3))))
 
@@ -376,15 +380,22 @@
 
     ;; Calling this field-list in the parse tree.
     (struct-declaration-list		; S 6.7.2.1
+     (struct-declaration-list-1 ($$ (tl->list $1)))
+     ;; GNU extension empty struct; we use "char _[];" where _ => *anon*
+     ($empty ($$ '(field-list (comp-decl
+                               (decl-spec-list (type-spec (fixed-type "char")))
+                               (comp-declr-list
+                                (comp-declr (ary-declr (ident "*anon*")))))))))
+    (struct-declaration-list-1
      (struct-declaration ($$ (make-tl 'field-list $1)))
+     (struct-declaration-list-1 struct-declaration ($$ (tl-append $1 $2)))
      (lone-comment ($$ (make-tl 'field-list $1)))
-     (struct-declaration-list struct-declaration ($$ (tl-append $1 $2)))
-     (struct-declaration-list lone-comment ($$ (tl-append $1 $2)))
+     (struct-declaration-list-1 lone-comment ($$ (tl-append $1 $2)))
      ;;(cpp-statement ($$ (make-tl 'field-list $1)))
      ;;(struct-declaration-list cpp-statement ($$ (tl-append $1 $2)))
      ;; Not in C99, but allowed by GNU, I believe:
      (";" ($$ (make-tl 'field-list)))
-     (struct-declaration-list ";" ($$ $1)))
+     (struct-declaration-list-1 ";" ($$ $1)))
 
     (struct-declaration			; S 6.7.2.1
      (struct-declaration-no-comment ";")
@@ -497,18 +508,34 @@
 
     (attr-expr-list
      (attr-expr-list-1 ($$ (tl->list $1))))
-
     (attr-expr-list-1
      (attribute-expr ($$ (make-tl 'attr-expr-list $1)))
      (attr-expr-list-1 "," attribute-expr ($$ (tl-append $1 $3))))
 
     (attribute-expr
      (type-name)
-     ($fixed ($$ `(fixed ,$1)))
      (string-literal)
-     (identifier)
-     (attr-word "(" attr-expr-list ")" ($$ `(attribute ,$1 ,$3)))) ;; ???
+     (attr-word "(" attr-expr-list ")" ($$ `(attribute ,$1 ,$3)))
+     (attr-additive-expr))
 
+    ;; a bit kludgy but want to minimalize need (see issue#25)
+    (attr-additive-expr
+     (attr-multiplicative-expr)
+     (attr-additive-expr "+" attr-multiplicative-expr ($$ `(add ,$1 ,$3)))
+     (attr-additive-expr "-" attr-multiplicative-expr ($$ `(sub ,$1 ,$3))))
+    (attr-multiplicative-expr
+     (attr-unary-expr)
+     (attr-multiplicative-expr "*" attr-unary-expr ($$ `(mul ,$1 ,$3)))
+     (attr-multiplicative-expr "/" attr-unary-expr ($$ `(div ,$1 ,$3)))
+     (attr-multiplicative-expr "%" attr-unary-expr ($$ `(mod ,$1 ,$3))))
+    (attr-unary-expr
+     (attr-primary-expr)
+     ("sizeof" "(" type-name ")" ($$ `(sizeof-type ,$3)))
+     ("_Alignof" "(" type-name ")" ($$ `(alignof-type ,$3))))
+    (attr-primary-expr
+     ($fixed ($$ `(fixed ,$1)))
+     (identifier))
+     
     ;; --- declarators
 
     (init-declarator-list		; S 6.7
@@ -795,7 +822,7 @@
      (":" asm-output ($$ (make-tl 'asm-outputs $2)))
      (asm-outputs-1 "," asm-output ($$ (tl-append $1 $3))))
     (asm-output
-     (string-literal "(" identifier ")" ($$ `(asm-operand ,$1 ,$3)))
+     (string-literal "(" expression ")" ($$ `(asm-operand ,$1 ,$3)))
      ("[" identifier "]" string-literal "(" identifier ")"
       ($$ `(asm-operand ,$2 ,$4 ,$6))))
     (asm-inputs
